;
;	FREADS.ASM
;
;	For Clipper Summer '87.  Like freadstr() but read terminates
;	on <Ctrl-Z> instead of null character.
;
;	David Rifkind
; BIX  drifkind

		extrn	__parinfo:far	; Get parameter info
		extrn	__parni:far	; Get an integer parameter
		extrn	__exmgrab:far	; Allocate memory
		extrn	__exmback:far	; Free memory
		extrn	__tread:far	; File read
		extrn	__retclen:far	; Return a string and length

		extrn	__retc:far	; Return null-terminated string
		extrn	ferror:far	; Return DOS error

		extrn	__TERROR:word	; Error code from __tread

FREADS_DATA	segment	word public '_DATA'
null_string	db	0		; String returned on error
FREADS_DATA	ends

DGROUP		group	FREADS_DATA

FREADS_TEXT	segment	byte public 'CODE'
		assume	cs:FREADS_TEXT,ds:DGROUP

read_len	equ	word ptr [bp-2]		; Actual bytes read
request_len	equ	word ptr [bp-4]		; Requested string length
buffer_seg	equ	word ptr [bp-6]		; Segment of string buffer
buffer_off	equ	word ptr [bp-8]		; Offset of string buffer
buffer_ptr	equ	dword ptr [bp-8]	; Buffer offset and segment
handle		equ	word ptr [bp-10]	; File handle
ferror_ptr	equ	word ptr [bp-12]	; Address of error variable

		public	freads
freads		proc	far

		push	bp		; Standard entry sequence
		mov	bp,sp
		sub	sp,12		; Reserve local space
		push	di

		mov	ax,seg ferror	; Find error variable by peeking
		mov	es,ax		;   into "ferror" routine
		mov	bx,word ptr es:ferror+2
		mov	ferror_ptr,bx
		mov	word ptr [bx],0	; Initially assume no error

		xor	ax,ax		; Check right number of parameters
		push	ax
		call	__parinfo
		add	sp,2
		cmp	ax,2
		jnz	fail_1

		mov	ax,1		; Check first parameter integer
		push	ax
		call	__parinfo
		add	sp,2
		test	al,2h
		jz	fail_1

		mov	ax,2		; Check second parameter integer
		push	ax
		call	__parinfo
		add	sp,2
		test	al,2h
		jz	fail_1

		mov	ax,1		; Get file handle
		push	ax
		call	__parni
		add	sp,2
		mov	handle,ax

		mov	ax,2		; Get requested string length
		push	ax
		call	__parni
		add	sp,2
		mov	request_len,ax

		push	ax		; Allocate a buffer for string
		call	__exmgrab
		add	sp,2
		mov	buffer_off,ax
		mov	buffer_seg,dx
		or	ax,dx		; Check that allocation succeeded
		jnz	freads_1

fail_1:		jmp	freads_fail

freads_1:	push	request_len	; Read requested length from file
		push	buffer_seg
		push	buffer_off
		push	handle
		call	__tread
		add	sp,8
		mov	read_len,ax	; Save actual size read
		mov	ax,__TERROR	; Set error code from read
		mov	bx,ferror_ptr
		mov	[bx],ax

		mov	cx,read_len	; Scan for <Ctrl-Z> in string
		les	di,buffer_ptr
		mov	al,1Ah
		repne	scasb
		jne	freads_2
		inc	cx		; <Ctrl-Z> found, CX is one too low
freads_2:	sub	cx,read_len	; String length = read_len - CX
		neg	cx
		push	cx
		push	es
		push	buffer_off
		call	__retclen	; Return string to Clipper
		add	sp,6

		push	request_len	; Free up the buffer
		push	buffer_seg
		push	buffer_off
		call	__exmback
		add	sp,6

		pop	di
		mov	sp,bp
		pop	bp
		retf

freads_fail:	lea	ax,null_string	; Failed, return null string
		push	ds
		push	ax
		call	__retc
		add	sp,4
		pop	di
		mov	sp,bp
		pop	bp
		ret

freads		endp

FREADS_TEXT	ends
		end

