* MemoPak2 is a Clipper UDF to pack the .DBT associated with a .DBF.
* This function requires the name of the DBF and a name to use for the
* temporary index that will be created. It only works on files with 1
* memo field and will lose and or corrupt all of the data associated
* with all memo fields other than the first.  The main advantage of
* this function over the more standard COPY TO; DELETE; RENAME, syntax,
* is that it uses less disk space and due to the use of EMPTY() to test
* for the existence of a memo field the packed file may end up smaller
* than the COPY TO method, which copies all memos which contain
* anything, even 1 space.  The disadvantage, at least in the piece of
* code is speed.  It seems to run 10% to 20% slower than the COPY TO
* method.
*
*   Ira Emus
*   irae  BIX
*   Sep. 18, 1988
*   memopak2("filename",'indxname')


FUNCTION memopak2

  PARAMETER file2pack,ntxname
  dbfname = TRIM(file2pack)+".dbf"
  dbtname = TRIM(file2pack)+".dbt"
  pkname = SPACE(10)
  memoname = SPACE(10)
  pkoffset = getmemoname(dbfname,@pkname)

  pkname = substr(pkname,1,at(chr(0),pkname)-1)
  memo2chr(dbfname,pkoffset)
  SELE 0
  USE (file2pack) ALIAS f2pack

  INDEX ON &pkname TO &ntxname
  *
  * Find the first memofield
  *
  SET SOFTSEEK ON
  SEEK "         1"

  dbtnum = FOPEN(dbtname,2)
  *
  * cur_pos is the current offset into the .dbt file where the next memofield
  * will be written.  The actual offset will be cur_pos * 512.
  *
  cur_pos = 1
  DO WHILE !EOF()
    *
    * The location of the memofield attached to the current record is
    * determined by looking at the contents of the memo field in the
    * database and multipling by 512

    position = FSEEK(dbtnum,(512*VAL(&pkname)),0)

    *
    * The length of the read, in this case, 2048, is the maximum
    * length memo field this will handle. It can be increased to 64K if
    * necessary, but the speed of this function will be impacted. Set as
    * short as possible for any given application. The use of very large
    * numbers for this may cause out of memory errors.
    *

    contents = freads(dbtnum,2048)

    if !EMPTY(contents)
      FSEEK(dbtnum,cur_pos*512,0)
      FWRITE(dbtnum,contents+CHR(26))
      REPLACE &pkname WITH STR(cur_pos,10)
      cur_pos = cur_pos+1+INT(LEN(contents)/512)
    ELSE
      REPLACE &pkname WITH SPACE(10)
    ENDIF
    SKIP
  ENDDO
  CLOSE DATA
  FSEEK(dbtnum,cur_pos*512,0)
	FWRITE(dbtnum,'',0)
	FSEEK(dbtnum,0,0)
 	FWRITE(L2BIN(cur_pos))
  FCLOSE(dbtnum)
  chr2memo(dbfname,pkoffset)
  RETURN .T.


FUNCTION getmemoname

  * This function will return the offset of the field type specifier for the
  * first memo field and will put the name of the field in the parameter
  * fieldname. The parameter fieldname MUST be passed by reference. The
  * string containing the field name will be padded with trailing CHR(0)'s
  * which must be removed before they may be used as a macro.
  *
  * offset = getmemoname(filename,@fieldname)
  *

  PARAMETER filename,fieldname
  handle = FOPEN(filename,2)
  test = FREADSTR(handle,1)
  IF "" = test
    FSEEK(handle,8,0)
    headlen = SPACE(2)
    FREAD(handle,@headlen,2)
    headlen = BIN2W(headlen)
    rectype = 43
    FSEEK(handle,rectype,0)
    test = FREADSTR(handle,1)
    DO WHILE test  # 'M' .AND. headlen > rectype
      rectype = rectype+ 32
      FSEEK(handle,rectype,0)
      test = FREADSTR(handle,1)
    ENDDO
    IF test = "M"
      FSEEK(handle,rectype-11,0)
      FREAD(handle,@fieldname,10)
      FCLOSE(handle)
      RETURN rectype
    ENDIF
  ENDIF
  RETURN -1

FUNCTION memo2chr

  * This function given an offset and a filename will change the specified
  * location to a "C" and change the header to indicate NO associated DBT.
  * The offset should have been obtained with getmemoname.
  *
  * memo2chr(filename,offset)
  *

  PARAMETER filename,offset
  handle = FOPEN(filename,2)
  test = FREADSTR(handle,1)
  IF "" = test
    FSEEK(handle,0,0)
    FWRITE(handle,"")
    FSEEK(handle,offset,0)
    FWRITE(handle,"C")
    FCLOSE(handle)
    RETURN .T.
  ENDIF
  RETURN .F.



FUNCTION chr2memo

  * This function given an offset and a filename will change the specified
  * location to a "M" and change the header to indicate that there is an
  * associated DBT.
  * The offset should have been obtained with getmemoname.
  *
  * chr2memo(filename,offset)
  *

  PARAMETER filename,offset
  handle = FOPEN(filename,2)
  FWRITE(handle,"")
  FSEEK(handle,offset,0)
  test = FREADSTR(handle,1)
  IF test = "C"
    FSEEK(handle,offset,0)
    FWRITE(handle,"M")
    FERROR()
    FCLOSE(handle)
    RETURN offset
  ENDIF
  RETURN -1
